#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>

// Software SPI (slower updates, more flexible pin options):
// pin 8 - Serial clock out (SCLK)
// pin 9 - Serial data out (DIN)
// pin 10 - Data/Command select (D/C)
// pin 11 - LCD chip select (CS) - not used
// pin 12 - LCD reset (RST)
Adafruit_PCD8544 display = Adafruit_PCD8544(8, 9, 10, 11, 12);

// Audio Spectrum Analyser
#define SampleInput A0   // Name the sample input pin
#define BandWidth  500   // BandWidth
#define MaxFreq   6000   // Max analysis frequency
#define Trigger     10   // Trigger to synchronise the sampler 2vpp at 1kHz = 32

// Define various ADC prescaler
const unsigned char PS_16=(1<<ADPS2);
const unsigned char PS_32=(1<<ADPS2)|(1<<ADPS0);
const unsigned char PS_64=(1<<ADPS2)|(1<<ADPS1);
const unsigned char PS_128=(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);

// Setup the serial port and pin 2
void setup() {
  // Setup the ADC
  pinMode(SampleInput,INPUT);
  ADCSRA&=~PS_128;  // Remove bits set by Arduino library
  // Set prescaler 
  // ADCSRA|=PS_64; // 64 prescaler (250 kHz assuming a 16MHz clock)
  // ADCSRA|=PS_32; // 32 prescaler (500 kHz assuming a 16MHz clock)
  ADCSRA|=PS_16;    // 16 prescaler (1 MHz assuming a 16MHz clock)

  display.begin();
  display.setContrast(70);
  display.clearDisplay();
  display.println("Spectrum");              // Project
  display.println("Analyser");
  display.println("0-20 kHz");              // Range
  display.println();
  display.println("agp.cooper");            // Author
  display.println("@gmail.com");
  display.display();
  delay(2000);
}

void loop() {  
  static byte *samples;           // Sample array pointer
  static byte *window;            // Window array pointer
  static int N=0;                 // Number of samples for BandWidth
  static long sampleFreq;         // Sample frequency     
  long freq;                      // Frequency of interest
  float s;                        // Goertzel variables
  float s_prev;
  float s_prev2;
  float coeff;
  float magn;
  int i;
  unsigned long ts;
  unsigned long tf;

  if (N==0) {
    // Set number of samples and check sample frequency
    N=123000/BandWidth+1;
    samples=(byte *)malloc(N);
    ts=micros();
    for (i=0;i<N;i++) samples[i]=(byte)(analogRead(SampleInput)>>2);
    tf=micros();
    free(samples);
    sampleFreq=1000000*N/(tf-ts);
    N=2*sampleFreq/BandWidth+1;
    
    // Create arrays
    samples=(byte *)malloc(N);
    window=(byte *)malloc(N);

    // Repeat sampleFrequency test
    ts=micros();
    for (i=0;i<N;i++) samples[i]=(byte)(analogRead(SampleInput)>>2);
    tf=micros();
    sampleFreq=1000000*N/(tf-ts);
    
    display.clearDisplay();                   // Clear display
    display.setCursor(0,0);                   // Set cursor to beginning
    display.print("SI: A");                   // Sample input pin 
    display.println(SampleInput-14);
    display.print("SF: ");                    // Sample frequency 
    display.println(sampleFreq);
    display.print("MF: ");                    // Max frequency 
    display.println(MaxFreq);
    display.print("BW: ");                    // BandWidth
    display.println(BandWidth);
    display.print("SN: ");                    // Number of samples
    display.println(N);
    display.display();                        // Display
    delay(2000);
    
 
    // Modified Hamming Window
    for (i=0;i<N;i++) window[i]=(byte)((0.54-0.46*cos(2*M_PI*i/(N-1)))*255); 

    // Generate test samples
    for (i=0;i<N;i++) {
      samples[i]=(byte)(30*(sin(2*M_PI*i*5000/sampleFreq)+sin(2*M_PI*i*2500/sampleFreq)+sin(2*M_PI*i*7500/sampleFreq)+sin(2*M_PI*i*10000/sampleFreq))+127);
    }
  }

  if (true) {
    // Sychronise the start of sampling with data slicer
    int a0,a1;
    a0=1023;
    for (i=0;i<N;i+=2) {
      a1=analogRead(SampleInput);
      a0=(a0*13+a1*3)/16;
      if (a1>a0+3) break;
    }
    ts=micros();
    for (i=0;i<N;i++) samples[i]=(byte)(analogRead(SampleInput)>>2);
    tf=micros();
    sampleFreq=1000000*N/(tf-ts);
  }
  
  // Process samples
  display.clearDisplay();                   // Clear display
  // Scan frequencies
  for (freq=0;freq<=MaxFreq;freq+=(MaxFreq/60)) {
    // Goertzel (https://en.wikipedia.org/wiki/Goertzel_algorithm)
    coeff=2*cos(2*M_PI*freq/sampleFreq);
    s_prev=0;
    s_prev2=0;
    for (i=0;i<N;i++) {
      s=0.0000768935*window[i]*samples[i]+s_prev*coeff-s_prev2;
      s_prev2=s_prev;
      s_prev=s;
    }
    // Get magnitude
    magn=2*sqrt(s_prev2*s_prev2+s_prev*s_prev-coeff*s_prev*s_prev2)/N;
    // Display on MicroView
    display.drawLine(freq*60/MaxFreq,47,freq*60/MaxFreq,10-(int)(20*log10(magn+0.0001)),BLACK);
  }
  // Frequency graduations
  display.setCursor(65,0);
  display.print(MaxFreq/1000);
  display.print("k");
  display.drawLine(0,0,0,5,BLACK);
  display.drawLine(10,0,10,2,BLACK);
  display.drawLine(20,0,20,5,BLACK);
  display.drawLine(30,0,30,2,BLACK);
  display.drawLine(40,0,40,5,BLACK);
  display.drawLine(50,0,50,2,BLACK);
  display.drawLine(60,0,60,5,BLACK);
  // Voltage graduations   
  display.drawLine(0,40,60,40,BLACK);
  display.setCursor(65,38);
  display.print("-30");
  display.drawLine(0,20,60,20,BLACK);
  display.setCursor(65,18);
  display.print("-10");
  display.drawLine(0,10,60,10,BLACK);
  display.setCursor(65,8);
  display.print("  0");
  //Display
  display.display();
}
